/*
*  Rubrica
*  file: import.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "file.h"
#include "main.h"
#include "error.h"
#include "import.h"
#include "import_int.h"

ImportFrom importfrom;

RubricaItem *r;
TelNum *tt;
GList *web=NULL, *mail=NULL, *tel=NULL;
GList *note;

/*
TokenCard token;
SubToken subtoken=SUB_BAD;
*/

GToken gtoken[] = { 
  {"BEGIN", BEGIN, TRUE},             {"FN", FN, FALSE},  
  {"N", N, FALSE},                    {"REV", REV, FALSE},  
  {"BDAY", BDAY, FALSE},              {"ADR", ADR, TRUE},
  {"TEL", TEL, TRUE},                 {"EMAIL", EMAIL, TRUE},  
  {"URL", URL, FALSE},                {"TZ", TZ, FALSE},
  {"GEO",GEO},                        {"TITLE", TITLE, FALSE},            
  {"ORG", ORG, FALSE},                {"CATEGORIES", CATEGORIES, FALSE},  
  {"NOTE", NOTE, FALSE},              {"KEY", KEY, TRUE}, 
  {"END", END, TRUE},                 {NULL, BAD, FALSE}
};

Subtoken subgtoken[] = {
  {"HOME", HOME},         {"WORK",WORK },             {"POSTAL",POSTAL}, 
  {"PARCEL",PARCEL },     {"DOM", DOM},               {"INTL",INTL },   
  {"PREF",PREF },         {"VOICE",VOICE },           {"FAX", FAX},
  {"MSG",MSG },           {"CELL",CELL },             {"PAGER",PAGER },
  {"BBS", BBS},           {"MODEM",MODEM },           {"CAR",CAR },      
  {"ISDN", ISDN},         {"VIDEO",VIDEO},            {"AOL",AOL }, 
  {"INTERNET",INTERNET }, {"APPLELINK",APPLELINK },   {"ATTMAIL", ATTMAIL}, 
  {"CIS", CIS},           {"EWORLD", EWORLD},         {"IBMMail",IBMMail },
  {"MCIMAIL",MCIMAIL },   {"POWERSHARE",POWERSHARE }, {"PRODIGY", PRODIGY},
  {"TLX", TLX},           {"X400",X400},              {"VCARD", VCARD},
  {"PGP",PGP},            {"X509",X509},
  {"QUOTED-PRINTABLE",QUOTED-PRINTABLE},              {NULL, SUB_END}
};		

KToken ktoken[] = {
  {"Entries",Entries},    {"address",address},    {"town",town},        
  {"country",country},    {"zip",zip},            {"birthday", birthday},
  {"comment",comment},    {"URL",URL},            {"email" email,},
  {"email2", email2},     {"email3",email3},      {"emails", emails},
  {"fax",fax},            {"modem",modem},        {"firstname",firstname},
  {"fn", fn},             {"name",name},          {"namePrefix",namePrefix},
  {"title",title},        {"keywords",keywords},  {"org",org},
  {"orgUnit",orgUnit},    {"role",role},          {"orgSubUnit",orgSubUnit},
  {"state",state},        {"talk",talk},          {"telephone",telephone},
  {"deliveryLabel",deliveryLabel},      
  {"additionalName",additionalName},
  {"END",END},            {"UNKNOW_TOK", UNKNOW_TOK}, 
  {"END_KTOK", END_KTOK}  
};


static gchar *gcard_default="/.gnome/GnomeCard.gcrd";
static gchar *card_kde1="/.kde/share/apps/kab/addressbook.database";
static gchar *card_kde2="";
gint ntel, nmail, n;
gint notelen;


void import_init_item(RubricaItem *r)
{
  r->editabile = r->cancellabile = TRUE;      

  r->scheda = r->nome = r->cognome = r->professione = r->via = NULL;
  r->numero = r->cap = r->citta = r->provincia = r->stato = r->note = NULL;

  r->web_list = web;
  r->mail_list = mail;
  r->tel_list = tel;      

  r->nweb = r->nmail = r->ntel = 0;
  note = NULL;  
}

void import_from_init(void)
{
  importfrom = CSV; /* import default: csv file */
}

void import_from_set(ImportFrom i)
{
  importfrom = (ImportFrom) i;
}

ImportFrom import_from_get(void)
{
  return (ImportFrom) importfrom;
}


void import_data(void)
{
  GtkWidget *fentry;
  gchar *filename;
  gchar *buffer, *titolo = NULL;
  
  n=0;
  fentry = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY
				      (rubrica_import_get_file_entry()));

  filename = gtk_entry_get_text(GTK_ENTRY(fentry));

  /* creao una nuova lista rubrica, e 
     salvo una eventuale lista precedente */
  rubrica_crea_nuovo_archivio();

  switch (importfrom)
    {
    case GCARD:
      /* 	 if no name is set in gnome_entry, then 
		 load default gnomecard addressbook 
      */
      if (strcmp(filename,"") == 0)
	{
	  /* carico il file di default */
	  buffer = g_get_home_dir();
	  
	  filename = g_strdup_printf("%s%s",buffer,gcard_default);
	  rubrica_import_open_file(filename);
	  titolo = _("Imported from GnomeCard");
	}          
      else	
	/* import gnomecard file */
	{
	  rubrica_import_open_file(filename);
	}
      break;
      
    case KCARD:
      /* kab (kde 1.x) e (kab 2.x) ha un database di default */
      buffer =  g_get_home_dir();
      
      filename = g_strdup_printf("%s%s",buffer,card_kde1);
      rubrica_import_open_file(filename);
      titolo = _("Imported from Kab");
      break;

    case RUBRICA:
      if (strcmp(filename, "") == 0)
	{
	  GtkWidget *app;
	  
	  app = rubrica_get_app();  
	  gnome_app_error(GNOME_APP(app), _("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else
      /* apri il file e leggilo */
	{
	  g_print("\nimport %s", filename);
	  n = rubrica_import_open_rubrica_old_file(filename);      
	}
      break;

    case CSV:
    default:
      if (strcmp(filename,"") == 0)
	{
	  GtkWidget *app;
	  
	  app = rubrica_get_app();  
	  gnome_app_error(GNOME_APP(app), _("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else 
	{
	  g_print("file: %s", filename);
	  rubrica_import_open_file(filename);      
	  titolo = _("Imported from Netscape cvs");	  
	}
      break;      
    }

  /* inizializzo nomefile e path (quando si salva 
     il file importato bisogna dargli un nome) */
  rubrica_file_init_names();

  /* visualizzo il numero di indirizzi importati */
  rubrica_vista_set_indirizzi(n);

  /* la rubrica  modificata */
  rubrica_modificata(TRUE);

  rubrica_vista_set_label(titolo);
  rubrica_set_empty(FALSE);  
}


void rubrica_import_open_file(gchar *file)
{
  FILE *fp;
//  gboolean esiste;

  if ((fp=fopen(file, "r")) == NULL)
    {
      /* messaggio di errore */ 
      g_print("\nerrore, impossibile leggere il file");
      errore_import_no_file();

      return;
    }

  while (!feof(fp))
    {
      gchar buffer[255];

      while (fgets(buffer, 255, fp))
	{	  
	  g_print("decodificare -- %s\n", buffer);
	  rubrica_decode_buffer(buffer);
	}
    }
}

void export_data(void)
{
}

void rubrica_decode_buffer(gchar *buffer)
{
  gchar *str=NULL;
  gchar *buf=NULL, *tmp=NULL, *notebuf, *orgbuf, *revbuf;
  gchar *decoded[8];
  TokenGCard token;
  SubGToken subtoken;
 
  str = g_strdup(buffer);
  str = strtok(str, ":");  
  token = decode_token(str, &subtoken);

  switch(token)
    {
    case BEGIN:
      /* allocare lo spazio per un nuovo indirizzo */
      r = (RubricaItem*) g_malloc(sizeof(RubricaItem));
      ntel = nmail = notelen = 0;
      import_init_item(r);
      break;
      
      /*
	case VCARD:
	non fare nulla,  compreso in begin 
	break;
      */

    case FN:
      /* nome della scheda */ 
      str = strtok(NULL, ":");
      r->scheda = g_strdup(str);      
      break;
      
    case N:
      /* ordine:: cognome, nome, secondo nome, prefisso, suffisso */       
      buf = import_get_string(buffer);
      import_decode_string(buf, decoded);
 
      r->cognome = g_strdup(decoded[0]);
      r->nome = g_strdup(decoded[1]);
      /*
	decoded[2] <= second name
	decoded[3] <= prefix
	decoded[4] <= suffix
       */
      if (decoded[2])
	{
	  tmp = g_strdup_printf(_("Second name: %s\n"),decoded[2]);
	  notelen += strlen(tmp);
	  note = g_list_append(note, tmp);
	}
      if (decoded[3])
	{
	  tmp = g_strdup_printf(_("Prefix: %s\n"),decoded[3]);
	  notelen += strlen(tmp);
	  note = g_list_append(note, tmp);     
	}
      if (decoded[4])
	{
	  tmp = g_strdup_printf(_("Suffix: %s\n"),decoded[4]);
	  notelen += strlen(tmp);
	  note = g_list_append(note, tmp);      
	}

      buf = NULL;
      break;

    case REV:
      /* data, fuso orario, ora */
      str = strtok(NULL, ";");
      revbuf = g_strdup_printf(_("Fuso orario: %s\n"),str);
      notelen += strlen(revbuf);
      note = g_list_append(note, revbuf);
      break;

    case BDAY:
      str = strtok(NULL, ":");
      buf = g_strdup_printf(_("Birthday: %s\n"),str);
      notelen += strlen(buf);
      note = g_list_append(note, buf);

      g_free(buf);
      buf = NULL;
      break;

    case ADR:
      /* indirizzo di posta + tipo */
      buf = import_get_string(buffer);
      import_decode_string(buf, decoded);

      r->via = g_strdup(decoded[2]);
      r->numero = g_strdup("");
      r->cap = g_strdup(decoded[5]);
      r->citta = g_strdup(decoded[3]);
      r->provincia = g_strdup(decoded[4]);
      r->stato = g_strdup(decoded[6]);
      /* 
	 decoded[0] <= post office
	 decoded[1] <= extended
	 decoded[4] <= region ==> province
      */
      if (decoded[0])
	{
	  tmp = g_strdup_printf(_("Post Office: %s\n"),decoded[0]);
	  notelen += strlen(tmp);
	  note = g_list_append(note, tmp);      
	}
      if (decoded[1])
	{
	  tmp = g_strdup_printf(_("Extended: %s\n"),decoded[1]);
	  notelen += strlen(tmp);
	  note = g_list_append(note, tmp);      
	}
      buf = NULL;
      break;

    case TEL:
      /* telefono + tipo */
      tt = (TelNum*) g_malloc(sizeof(TelNum));

      switch (subtoken)
	{	  
	case HOME:
	  str = import_get_string(buffer);
	  tt->numero_telefono = g_strdup(str);
	  tt->tipo = "Casa";
	  tel = g_list_append(tel, tt);
	  ntel++;
	  break;
	case WORK:
	  str = import_get_string(buffer);
	  tt->numero_telefono = g_strdup(str);
	  tt->tipo = "Ufficio";
	  tel = g_list_append(tel, tt);
	  ntel++;
	  break;
	case FAX:
	  str = import_get_string(buffer);
	  tt->numero_telefono = g_strdup(str);
	  tt->tipo = "Fax";
	  tel = g_list_append(tel, tt);	  
	  ntel++;
	  break;
	case CELL:
	  str = import_get_string(buffer);
	  tt->numero_telefono = g_strdup(str);
	  tt->tipo = "Cellulare";
	  tel = g_list_append(tel, tt);
	  ntel++;
	  break;
	  
	  /* altro */
	case PREF:
	case VOICE:
	case MSG:
	case PAGER:
	case BBS:
	case MODEM:
	case CAR:
	case ISDN:
	case VIDEO:
	default:
	  str = import_get_string(buffer);
	  tt->numero_telefono = g_strdup(str);
	  tt->tipo = "Altro";
	  tel = g_list_append(tel, tt);	  
	  ntel++;
	  break;
	} 
      break;
      
    case EMAIL:      
      /* email +tipo */
      switch(subtoken)
	{
	case AOL:
	case INTERNET:
	case APPLELINK:
	case ATTMAIL:
	case CIS:
	case EWORLD:
	case IBMMail:
	case MCIMAIL:
	case POWERSHARE:
	case PRODIGY:
	case TLX:
	case X400:
	default:
	  buf = g_strdup(import_get_string(buffer));
	  if (buf)
	    {
	      mail = g_list_append(mail, buf);	  
	      nmail++;
	    }
	  break;
	} /* switch subtoken */
      break;

    case URL:
      buf = import_get_string(buffer);
      if (buf)
	buf = g_strdup_printf("http:%s",buf);
	
      web = g_list_append(web, buf);      
      r->web_list = web;
      r->nweb = 1;
      /* url */
      break;
      
    case TZ:
      /* Time Zone */
      str = strtok(NULL, ":");
      tmp = g_srtdup_printf("Time Zone: %s\n", str);
      notelen += strlen(orgbuf);
      note = g_list_append(note, tmp);     
      break;

    case GEO:
      /* Geo Location (latitude, longitude) */
      /* load: long then latit 
         es. lati -0,3 long 5,4 ==> 5,4,-0,3 */
      str = strtok(NULL, ":");
      tmp = g_srtdup_printf("Time Zone: %s\n", str);
      notelen += strlen(orgbuf);
      note = g_list_append(note, tmp);           
      break;

    case TITLE:
      /* titolo nell'organizzazione (professione) */
      str = strtok(NULL, ":");
      r->professione = g_strdup(str);
      break;

    case ORG:
      /* nome dell'organizzazione */
      str = strtok(NULL, ":");
      orgbuf = g_strdup_printf(_("Org. Name: %s\n"),str);
      notelen += strlen(orgbuf);
      note = g_list_append(note, orgbuf);
      break;

    case CATEGORIES:
      /* quoted-printable: ignored */
      str = strtok(NULL, ":");
      notebuf = g_strdup_printf(_("Categories: %s\n"),str);     
      notelen +=strlen(notebuf);
      note = g_list_append(note, notebuf);      
      break;

    case NOTE:      
      str = strtok(NULL, ":");
      notebuf = g_strdup_printf(_("Comment: %s"),str);     
      notelen +=strlen(notebuf);
      note = g_list_append(note, notebuf);
      
      /* note */
      break;

    case KEY:
      /* public key */
      str = strtok(NULL, ":");
      switch (subtoken)
	{
	case PGP:
	  tmp = g_strdup_printf("Public Key (PGP):\n%s", str);
	  notelen += strlen(tmp);
	  note = g_list_append(note, buf);	  
	  break;

	case X509:
	  tmp = g_strdup_printf("Public Key (X509):\n%s", str);
	  notelen += strlen(tmp);
	  note = g_list_append(note, buf);	  	  
	  break;

	default:
	  break;
	}
      
      break;

    case END:
      /* collegare le liste mail e tel */
      r->mail_list = mail;
      r->tel_list = tel;
      r->nmail = nmail;
      r->ntel = ntel;
      r->note = g_strdup(import_decode_note(note));
      import_free_note(note);
      /* inserire l'elemento nella lista rubrica */
      rubrica_inserisci_elemento(r);
      n++;
      /* impostare il flag di lista vuota a falso */
      /* visualizzo la nuova scheda in lista */
      rubrica_inserisci_scheda(r);      
      break;

    case VCARD:  
    case BAD:
    default:
      break;
    }
}

TokenCard decode_token(gchar *str, SubGToken *st)
{
  gchar *tmp;
  gboolean decoded = FALSE;
  gint i=0, size = 0;

  size = sizeof(gtoken) / sizeof (gtoken[0]);

  while (!decoded && i < size)
    {
      if ((strcmp(str, gtoken[i].str)) == 0)
	{
	  if(gtoken[i].subtok)
	    {
	      tmp = g_strdup(str);
	      tmp = strtok(tmp,";");

	      *st = decode_subtoken(tmp);
	      g_free(tmp);
	    }

	  decoded = TRUE;
	  return token[i].tok;
	}
      else
	i++;
    }

  return (TokenCard) BAD;
}

SubToken decode_subtoken(gchar *str)
{
  gchar *tmp;
  gboolean decoded = FALSE;
  gint i=0; size = 0;

  size = sizeof(subgtoken) / sizeof (subgtoken[0]);
  
  while (!decoded && i < size)
    {
      if ((strcmp(str, subgtoken[i].str) == 0))	
	decoded = TRUE; 	  	 
      else
	i++;
    }

  if (decoded)
    return (SubGToken) subgtoken[i].stok;
  
  return (SubGToken) SUB_BAD;;
}

gchar *import_get_string(gchar *buf)
{     
  gchar *str;

  g_return_val_if_fail( buf != NULL, NULL);
  
  g_strreverse(buf);
  str = strtok(buf, ":");
  g_strreverse(str);
  
  return (gchar*) str;	      
}

void import_decode_string(char *buf, gchar *dec[7])
{
  gint i, j, k;
  gchar *tmp;

  /* inizializzazione buffer decodificato */
  for(i=0; i<8;i++)
    dec[i]=NULL;

  tmp = g_malloc (strlen(buf));
  i = 0;
  j = 0;
  k = 0;
  while (buf[j] != '\0')
    {
      if(buf[j] != ';')
	{
	  tmp[k] = buf[j];
	  j++;
	  k++;
	}	 
      else
	{
	  tmp[k]='\0';
	  dec[i] = g_strdup(tmp);
	  i++;
	  j++;
	  k=0;
	}		
    }
  tmp[k]='\0';
  dec[i] = g_strdup(tmp); 
}


gchar *import_decode_note(GList *l)
{
  gchar *nt, *tmp;


  nt = (gchar *) g_malloc(sizeof(gchar)*(notelen+1));
  *nt = '\0';
  while(note)
    {
      tmp = (gchar*) note->data;     
      strncat(nt, tmp, strlen(tmp));
      note = g_list_next(note);
    }   

  return (gchar *) nt;
}

void import_free_note(GList *n)
{
  gchar *tmp;
  GList *nodo;

  while (n)
    {
      nodo = g_list_last(note);
      n = g_list_remove_link(n, nodo);
      
      tmp = nodo->data;
      if (tmp)
	free(tmp);
      
      g_list_free_1(nodo);      
    } 
  n = NULL;
}

gint rubrica_import_open_rubrica_old_file(gchar *fname)
{
  rubrica_apri_old_file(fname);

  if (rubrica_is_empty())
    return 0;
  
  return (gint) rubrica_vista_get_indirizzi();    
}


