/*
*  Rubrica
*  file: prefer.c
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "prefer.h"
#include "error.h"
#include "rubrica.h"

RubricaPreferences *pref=NULL;
static gchar *rubricarc="rubrica.conf";
GnomePropertyBox *pbox;

void rubrica_preferences_init()
{  
  gchar *tmp;
  
  tmp = g_get_home_dir();  
  pref = (RubricaPreferences*) g_malloc(sizeof(RubricaPreferences));

  g_print("\nTemporary preferences routines. Rubrica will go to GConf soon.");

  pref->show_file = TRUE;
  pref->default_addressbook = TRUE;
  pref->alpha = TRUE;
  pref->date = FALSE;
  pref->increasing = TRUE;
  pref->confirm = TRUE;
  pref->backup = TRUE;

  pref->default_path = g_strdup_printf("%s/.rubrica/",tmp);
  pref->default_filename = g_strdup("rubrica.rub");

  pref->user_path = NULL;
  pref->user_filename = NULL;

  if (!g_file_exists(pref->default_path))
    {
      gint err;
      g_print("\nrubrica initialization...");      
      err = mkdir(pref->default_path, S_IRUSR | S_IWUSR | S_IXUSR);
      g_print("\nmakedir: %s\terr: %d", pref->default_path, err);
      
      rubrica_preferences_save();
    }
  else
    rubrica_preferences_load(pref);
}

void rubrica_pref_set_file_show(gboolean show_file)
{
  pref->show_file = show_file;
}

void rubrica_pref_set_default_addressbook(gboolean default_ab)
{
  pref->default_addressbook = default_ab;
}

void rubrica_pref_set_alpha_order(gboolean ord)
{
  if (ord)
    {
      pref->alpha = TRUE;
      pref->date = FALSE;
    }
  else
    {
      pref->alpha = FALSE;
      pref->date = TRUE;      
    }
}

void rubrica_pref_set_date_order(gboolean ord)
{
  if (ord)
    {
      pref->alpha = FALSE;
      pref->date = TRUE;
    }
  else
    {
      pref->alpha = TRUE;
      pref->date = FALSE;      
    }  
}

void rubrica_pref_set_exit_confirm(gboolean confirm)
{
  pref->confirm = confirm;
}

void rubrica_pref_set_increasing(gboolean inc)
{
  if (inc)
    pref->increasing = TRUE;
  else
    pref->increasing = FALSE;  
}

void rubrica_pref_set_do_backup(gboolean bk)
{
  if (bk)
    pref->backup = TRUE;
  else
    pref->backup = FALSE; 
}

gint rubrica_preferences_load(RubricaPreferences *p)
{
  FILE *fp;
  gchar *rubconf;

  rubconf = g_strdup_printf("%s%s",pref->default_path, rubricarc);
  if ((fp = fopen(rubconf, "r"))== NULL)
    {
      g_warning("\nI can't open configuration option. Run in default mode\n");
      
      return -1;
    }
  else 
    {
      gchar buffer[255];

      fscanf(fp,"%s",buffer); /* [rubrica] */
      fscanf(fp,"%s",buffer);
      p->show_file = prefer_get_bool(buffer);
      fscanf(fp,"%s",buffer);
      p->default_addressbook = prefer_get_bool(buffer);
      fscanf(fp,"%s",buffer);
      p->alpha = prefer_get_bool(buffer);
      fscanf(fp,"%s",buffer);
      p->date = prefer_get_bool(buffer);
      fscanf(fp,"%s",buffer);
      p->increasing = prefer_get_bool(buffer);
      fscanf(fp,"%s",buffer);
      p->confirm = prefer_get_bool(buffer);
      fscanf(fp,"%s",buffer);
      p->backup = prefer_get_bool(buffer);

      fscanf(fp,"%s",buffer);
      p->default_path = prefer_get_str(buffer);
      fscanf(fp,"%s",buffer);
      p->default_filename = prefer_get_str(buffer);
      fscanf(fp,"%s",buffer);
      p->user_path = prefer_get_str(buffer);
      fscanf(fp,"%s",buffer);
      p->user_filename = prefer_get_str(buffer);
    }

  /* apply loaded preferences */
  return 0;
}

gint rubrica_preferences_save()
{
  FILE *fp;
  gchar *rubconf;

  rubconf = g_strdup_printf("%s%s",pref->default_path, rubricarc);
  
  if ((fp = fopen(rubconf, "w"))== NULL)
    {
      g_error("\nI can't save configuration option\n");
      error_save_config("\nWarning, i can't save configuration option");
    }
  else 
    {
      fprintf(fp,"[Rubrica]\n");
      fprintf(fp,"SHOW_FILE=%d\n", pref->show_file); 
      fprintf(fp,"DEFAULT_ADDRESSBOOK=%d\n", pref->default_addressbook); 
      fprintf(fp,"ALPHA=%d\n", pref->alpha); 
      fprintf(fp,"DATE=%d\n", pref->date); 
      fprintf(fp,"INCREASING=%d\n", pref->increasing); 
      fprintf(fp,"CONFIRM=%d\n", pref->confirm);
      fprintf(fp,"BACKUP=%d\n", pref->backup);
      fprintf(fp,"PATH=%s\n", pref->default_path);
      fprintf(fp,"FILENAME=%s\n", pref->default_filename);    
      fprintf(fp,"USER_PATH=%s\n", pref->user_path);
      fprintf(fp,"USER_FILENAME=%s\n", pref->user_filename);      
    }
  fflush(fp);
  fclose(fp);
  return 0;
}

RubricaPreferences *rubrica_get_preferences()
{
  return (RubricaPreferences *) pref;
}

gboolean prefer_get_bool(gchar *buf)
{
  gchar *tmp;

  tmp = strtok(buf,"=");
  if((strcmp(buf,"1")) == 0)
    return TRUE;
  /* else */
  return FALSE;
}

gchar *prefer_get_str(gchar *buf)
{
  gchar *tmp;

  tmp = strtok(buf,"=");
  tmp = strtok(NULL, "");
  
  return (gchar *) g_strdup(tmp);
}

gint prefer_get_int(gchar *buf)
{
  gchar *tmp;

  tmp = strtok(buf,"=");
  tmp = strtok(NULL, "");
  
  return atoi(tmp);
}
