/*
*  Rubrica
*  file: rubrica.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"


GList *rubrica = NULL;
VistaDati vista;
guint scheda_corrente = 0;
gboolean mostra_path = FALSE;
gboolean modificata = FALSE;
gboolean rubrica_empty = TRUE;



GList *rubrica_get_rubrica(void)
{
  return (GList*) rubrica;
}

void rubrica_set_rubrica(gpointer ptr)
{
  rubrica = ptr;
}

void rubrica_init_vista(GtkWidget *label, GtkWidget *schede,  
			GtkWidget *dati, GtkWidget *pbar, gchar *titolo)
{
  vista.schede = schede;
  vista.dati = dati;
  vista.progressbar = pbar;
  vista.label_nomefile = label;

  vista.schedeinfo.riga = 0;
  vista.schedeinfo.colonna = 0;
  vista.schedeinfo.selezionato = FALSE;

  vista.totale_indirizzi = 0;

  rubrica_set_titolo(titolo);
}

void rubrica_get_vista(VistaDati *v)
{
  v->schede = vista.schede;
  v->dati = vista.dati;
  v->progressbar = vista.progressbar;
 
  v->label_nomefile = vista.label_nomefile;
 
  v->schedeinfo.riga = vista.schedeinfo.riga ;
  v->schedeinfo.colonna = vista.schedeinfo.colonna;
  v->schedeinfo.selezionato = vista.schedeinfo.selezionato;

  v->datiinfo.riga = vista.datiinfo.riga;
  v->datiinfo.colonna = vista.datiinfo.colonna ;
  v->datiinfo.selezionato = vista.datiinfo.selezionato ;

  v->totale_indirizzi = vista.totale_indirizzi ;
}

void
rubrica_inserisci_scheda(RubricaItem *r)
{
  gchar *buffer[5];
  
  g_return_if_fail(r != NULL);

  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(r->scheda);
  buffer[2] = g_strdup(r->nome);
  buffer[3] = g_strdup(r->cognome);
  buffer[4] = NULL;

  gtk_progress_set_value(GTK_PROGRESS(vista.progressbar), 
			 (float) vista.totale_indirizzi);
  gtk_clist_append(GTK_CLIST(vista.schede), buffer);
}

void 
rubrica_elimina_scheda(gint n)
{
  /* il numero degli elementi  stato aggiornato nella 
     chiamata a rubrica_elimina_elemento
  */
  gtk_clist_remove(GTK_CLIST(vista.schede), n);  
  gtk_progress_set_value(GTK_PROGRESS(vista.progressbar), 
			 (float) vista.totale_indirizzi);  
}

void
rubrica_modifica_scheda(RubricaItem *r)
{
  gchar *buffer[5];
  gint n;

  g_return_if_fail(r != NULL);

  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(r->scheda);
  buffer[2] = g_strdup(r->nome);
  buffer[3] = g_strdup(r->cognome);
  buffer[4] = NULL;
  
  n = vista.schedeinfo.riga;

  gtk_clist_remove(GTK_CLIST(vista.schede), n);
  gtk_clist_insert(GTK_CLIST(vista.schede), n, buffer);
}

void rubrica_set_titolo(gchar *titolo)
{
  g_return_if_fail(titolo != NULL);

  if (strlen(titolo) > 20)
    {
      strncpy(vista.titolo, titolo, 20);
      vista.titolo[21] = '\0';
    }
  else
    strcpy(vista.titolo, titolo);
}

gchar *rubrica_get_titolo(void)
{
  return (gchar*) vista.titolo;
}


/* ************************************************************* */

/*  gestione clist
*/
GtkCList *rubrica_vista_get_schede(void)
{
  return GTK_CLIST(vista.schede);
}

GtkCList *rubrica_vista_get_dati(void)
{
  return GTK_CLIST(vista.dati);
}


/*  gestione label
*/
void rubrica_vista_set_label(gchar *str)
{
  gtk_label_set_text(GTK_LABEL(vista.label_nomefile), str);
}

gchar *rubrica_vista_get_label(void)
{
  gchar *str;

  gtk_label_get(GTK_LABEL(vista.label_nomefile), &str);
  return g_strdup(str);
}

/*   gestione info
*/
void rubrica_vista_set_info(InfoRubrica i, TIPO_VISTA t)
{
  switch (t)
    {
    case VISTA_SCHEDE:
      vista.schedeinfo.riga = i.riga;
      vista.schedeinfo.colonna = i.colonna;
      vista.schedeinfo.selezionato = i.selezionato;
      break;
	
    case VISTA_DATI:
      vista.datiinfo.riga = i.riga;
      vista.datiinfo.colonna = i.colonna;
      vista.datiinfo.selezionato = i.selezionato;  
      break;
	
    default:
      break;
    }
}

void rubrica_vista_get_info(InfoRubrica *i, TIPO_VISTA t)
{
  switch (t)
    {
      case VISTA_SCHEDE:
      i->riga = vista.schedeinfo.riga;
      i->colonna = vista.schedeinfo.colonna;
      i->selezionato = vista.schedeinfo.selezionato;  
      break;
    case VISTA_DATI:
      i->riga = vista.datiinfo.riga;
      i->colonna = vista.datiinfo.colonna;
      i->selezionato = vista.datiinfo.selezionato; 
      break;
    default:
      break;
    }
}

/*  gestione indirizzi
*/
inline gint rubrica_vista_get_indirizzi(void)
{
  return (gint) vista.totale_indirizzi;
}

inline void rubrica_vista_set_indirizzi(gint n)
{
  vista.totale_indirizzi = n;

  gtk_progress_set_value(GTK_PROGRESS(vista.progressbar), 
			 (float) vista.totale_indirizzi);
}

inline void rubrica_vista_inc_indirizzi(void)
{
  vista.totale_indirizzi++;
}

inline void rubrica_vista_dec_indirizzi(void)
{
  vista.totale_indirizzi--;
}

inline void rubrica_set_scheda_corrente(gint n)
{
  scheda_corrente = n;
}

inline gint rubrica_get_scheda_corrente(void)
{
  return (gint) scheda_corrente;
}

/*  ----------------------------------------------------------------- */

/* gestione elementi
*/
inline gboolean rubrica_is_modificata()
{
  return (gboolean) modificata;
}


void rubrica_modificata(gboolean stato)
{
  GtkWidget *win;

  win = rubrica_get_app();

  if (stato) /* modificata */
    {
/*
      gtk_window_set_title(GTK_WINDOW(win), _("Rubrica (Modificata)"));
      rubrica_set_titolo(_("Rubrica (Modificata)"));        
*/
      gtk_window_set_title(GTK_WINDOW(win), _("Rubrica (changed)"));
      rubrica_set_titolo(_("Rubrica (changed)"));        
    }
  else      /* non modificata */
    {
      /*
	gtk_window_set_title(GTK_WINDOW(win), _("Rubrica"));
	rubrica_set_titolo(_("Rubrica"));              
      */
      gtk_window_set_title(GTK_WINDOW(win), _("Rubrica"));
      rubrica_set_titolo(_("Rubrica"));         
    }
  
  modificata = stato;
}

void rubrica_inserisci_elemento(RubricaItem *r)
{
  rubrica = g_list_append(rubrica, r);
}

RubricaItem *rubrica_prendi_elemento(gint n)
{
  return (RubricaItem *) g_list_nth_data(rubrica, n);
}

gint rubrica_modifica_elemento(RubricaItem *r)
{
  gint n;
  GList *e = NULL;  
  gpointer new, old;

  g_return_val_if_fail(r != NULL, 0);

  new = r;
  n = vista.schedeinfo.riga;

  e = g_list_nth(rubrica, n);
  old = g_list_nth_data(rubrica,n);
  
  e->data = new;
  g_free(old);
  
  return 1;
}


void rubrica_free_data(gpointer data, gpointer user_data)
{  
  g_return_if_fail(data != NULL);
  g_return_if_fail(user_data != NULL);
  
  if (user_data)
    g_free(user_data);
}

gint rubrica_elimina_elemento(gint n)
{
  /*
    NOTA:
    La memoria occupata dalle liste si libera
    con g_list_free_1 e non con g_free() 
    pena SEGMENTATION FAULT
  */
  GList *nodo;
  GList *wl, *ml, *tl;
  RubricaItem *r;

  g_return_val_if_fail(rubrica != NULL, 0);

  /* prendo l'elemento alla posizione n */
  nodo = g_list_nth(rubrica, n);
  
  /* rimuovo l'elemento nodo dalla lista
     la memoria occupata da nodo ancora non  liberata. 
     Ora nodo  la testa di una nuova lista, posso aggiungere 
     altri elementi e costruire una nuova lista,
     o cancellarlo elimindolo dalla lista
  */
  rubrica = g_list_remove_link(rubrica, nodo);

  /* estraggo i dati contenuti nel nodo e libero 
     la memoria che occupano 
  */
  r = (RubricaItem*) g_list_nth_data(nodo, 0);

  /*
    r  un ptr a RubricaItem, 
    che contiene diversi *gchar e 3 liste.   
  */
  wl = r->web_list;
  ml = r->mail_list;
  tl = r->tel_list;

  /* ridondanza di controlli??
  	 preferisco effettuare pi controlli ed evitare
	 segmentation fault. Se qualcuno vuole reimplemantare
	 in modo migliore questa parte di codice,  il
	 benvenuto!! 

	 If someone thinks that there are much controls, 
	 well i think so. But i prefer this to a segmentation fault.
	 Enyone can modify this code, to make it better. Thanks :-)	 

   */
  if (wl) {
    g_list_foreach(wl, rubrica_free_data, wl->data);
    g_list_free_1(r->web_list); 
  }
  
  if (ml) {
    g_list_foreach(ml, rubrica_free_data, ml->data);
    g_list_free_1(r->mail_list);
  }
  
  if (tl) {
    g_list_foreach(tl, rubrica_free_data, tl->data);
    g_list_free_1(r->tel_list);
  }
  
  if(r->scheda)
    g_free(r->scheda);
  if(r->nome)
    g_free(r->nome);
  if(r->cognome)
    g_free(r->cognome);
  if(r->professione)
    g_free(r->professione);
  if(r->via)
    g_free(r->via);
  if(r->numero)
    g_free(r->numero);
  if(r->cap)
    g_free(r->cap);
  if(r->citta)
    g_free(r->citta);
  if(r->provincia)
    g_free(r->provincia);
  if(r->stato)
    g_free(r->stato);
  if(r->note)
    g_free(r->note);

  g_free(r);

  /* libero la menoria occupata da nodo */
  g_list_free_1(nodo);

  /* decremento il numero di elementi delle lista */
  rubrica_vista_dec_indirizzi();

  return 1;
}

void rubrica_svuota_memoria()
{
  gint n;
  
  n = g_list_length(rubrica);
  
  while (n)
    {
      rubrica_elimina_elemento(n-1);
      n--;
    }      
  rubrica_set_rubrica(NULL);
}


/*  ----------------------------------------------------------------- */

/* gestione configurazione 
*/
/*
void  rubrica_configura()
{
*/
 /* TO DO */ 
/*
}
*/
/*  ----------------------------------------------------------------- */



/* gestione visualizzazione dati schede
*/
void rubrica_mostra_scheda(RubricaItem *i)
{
  GtkCList *clist;
  GList *alias = NULL;
  gint n = 11;
  gint elementi = 0;
  gboolean modificato;              

  /* dati[..][0]  riservato per probabili future immagini
     dati[..][0] is reserved. Probably future pixmap
   */
  gchar *dati[18][4] = {
/*
    {NULL, _("Scheda: "),             NULL, NULL},
    {NULL, _("Nome: "),               NULL, NULL},
    {NULL, _("Cognome: "),            NULL, NULL},
    {NULL, _("Professione: "),        NULL, NULL},
    {NULL, _("Via: "),                NULL, NULL},
    {NULL, _("Numero: "),             NULL, NULL},
    {NULL, _("Cap: "),                NULL, NULL},
    {NULL, _("Citt: "),              NULL, NULL},
    {NULL, _("Provincia: "),          NULL, NULL},
    {NULL, _("Stato: "),              NULL, NULL},
    {NULL, _("Pagina Web: "),         NULL, NULL},
    {NULL, _("Indirizzo Email: "),    NULL, NULL},
    {NULL, _("Telefono Casa: "),      NULL, NULL},
    {NULL, _("Telefono Ufficio: "),   NULL, NULL},
    {NULL, _("Telefono Fax: "),       NULL, NULL},
    {NULL, _("Telefono Cellulare: "), NULL, NULL},
    {NULL, _("Note: "),               NULL, NULL},
    {NULL, NULL,                      NULL, NULL}
*/
    {NULL, _("Card: "),              NULL, NULL},
    {NULL, _("Name: "),              NULL, NULL},
    {NULL, _("Surname: "),           NULL, NULL},
    {NULL, _("Profession: "),        NULL, NULL},
    {NULL, _("Street: "),            NULL, NULL},
    {NULL, _("Number: "),            NULL, NULL},
    {NULL, _("Zip code: "),          NULL, NULL},
    {NULL, _("City: "),              NULL, NULL},
    {NULL, _("Province: "),          NULL, NULL},
    {NULL, _("State: "),             NULL, NULL},
    {NULL, _("Web Pages: "),         NULL, NULL},
    {NULL, _("Email Address: "),     NULL, NULL},
    {NULL, _("Home Thelephone: "),   NULL, NULL},
    {NULL, _("Work Thelephone: "),   NULL, NULL},
    {NULL, _("Fax Thelephone: "),    NULL, NULL},
    {NULL, _("Cellphone: "),         NULL, NULL},
    {NULL, _("Notes: "),             NULL, NULL},
    {NULL, NULL,                     NULL, NULL}
  };
  
  g_return_if_fail(i != NULL);

  clist = rubrica_vista_get_dati();
  gtk_clist_clear(GTK_CLIST(clist));

  dati[0][2] =  g_strdup(i->scheda);
  dati[1][2] =  g_strdup(i->nome);
  dati[2][2] =  g_strdup(i->cognome);
  dati[3][2] =  g_strdup(i->professione);
  dati[4][2] =  g_strdup(i->via);
  dati[5][2] =  g_strdup(i->numero);
  dati[6][2] =  g_strdup(i->cap);
  dati[7][2] =  g_strdup(i->citta);
  dati[8][2] =  g_strdup(i->provincia);
  dati[9][2] =  g_strdup(i->stato);

  rubrica_visualizza_dati(clist, dati);

  /*   web   */
  alias = g_list_first(i->web_list);
  elementi = i->nweb;
  modificato = FALSE;
  while (elementi > 0)
    {
      dati[10][2] = g_strdup(alias->data);  

      gtk_clist_insert(GTK_CLIST(clist),n, dati[10]);
      if (!modificato)
	{
	  dati[10][1] = g_strdup("");
	  modificato = TRUE;
	}
      n++;
      elementi--;
      alias = g_list_next(alias);
    }

  /*   mail   */
  alias = g_list_first(i->mail_list);
  modificato = FALSE;
  elementi = i->nmail;
  while (elementi > 0)
    {
      dati[11][2] = g_strdup(alias->data);  

      gtk_clist_insert(GTK_CLIST(clist),n, dati[11]);
      if (!modificato)
	{
	  dati[11][1] = g_strdup("");
	  modificato = TRUE;
	}
      n++;
      elementi--;
      alias = g_list_next(alias);
    }

  /*   telefono   */
  alias = g_list_first(i->tel_list);
  elementi = i->ntel;
  while (elementi > 0)
    {
      TelNum *tel = (TelNum*) alias->data;
      TIPO_TELEFONO tipo;
      
      tipo = tipo_telefono_int(tel->tipo);
      
      switch (tipo)
        {
        case TELEFONO_CASA:	  
          dati[12][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[12]);
          n++;
	  elementi--;
/*        dati[12][1] = "";  */
	  alias = g_list_next(alias);
          break;
        case TELEFONO_UFFICIO:
          dati[13][2] =  g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[13]);
          n++;
	  elementi--;
/*	  dati[13][1] = "";   */
	  alias = g_list_next(alias);
          break;
        case TELEFONO_FAX:
          dati[14][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[14]);
          n++;
	  elementi--;
/*	  dati[14][1] = "";     */
	  alias = g_list_next(alias);
          break;
        case TELEFONO_CELLULARE:
          dati[15][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[15]);
          n++;
	  elementi--;
/*	  dati[15][1] = "";         */
	  alias = g_list_next(alias);
          break;
        default:
          break;
        }
    }  
  dati[16][2] = g_strdup(i->note);
  gtk_clist_insert(GTK_CLIST(clist), n, dati[16]);  
}


/*   mostra i dati contenuti nella tabella "dati"
     dalla riga 0 alla riga 10
*/
void rubrica_visualizza_dati(GtkCList *clist, gchar *item[16][4])
{
  gint i;

  for (i=0; i<10; i++)
    gtk_clist_insert(GTK_CLIST(clist),i, item[i]);
}


/*  ----------------------------------------------------------------- */

/* 
   routine per la nagivazione delle schede inserite
   visualizzano la:
    - scheda precedente           (click pre)
    - scheda successiva           (click succ)
    - ultima scheda               (click prima)
    - prima scheda                (click ultima)
*/
void rubrica_mostra_prima_scheda()
{
  RubricaItem *i;
  GtkCList *clist;

  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(vista.totale_indirizzi > 0);

  i = g_list_nth_data(rubrica,0);
  
  g_return_if_fail(i != NULL);
  
  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), 0,0);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(0);
}

void rubrica_mostra_ultima_scheda()
{
  guint n;
  RubricaItem *i;
  GtkCList *clist;
  InfoRubrica info;
  
  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(vista.totale_indirizzi > 0);

  /* se la scheda contiene n elementi, l'ultimo elemento
     della lista  in posizione n-1.
     es. 
     posizione :  0  1  2  3
     elemento  :  1  2  3  4
  */
  n = g_list_length(rubrica)-1;
  i = g_list_nth_data(rubrica,n);

  g_return_if_fail(i != NULL);

  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), info.riga, info.colonna);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(n);
}


void rubrica_scheda_precedente()
{
  guint n;
  GtkCList *clist;
  RubricaItem *i;
  InfoRubrica info;

  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  n = rubrica_get_scheda_corrente()-1;
  
  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(vista.totale_indirizzi > 0);
  g_return_if_fail(n >= 0);

  i = g_list_nth_data(rubrica,n);
  
  g_return_if_fail(i != NULL);

  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), info.riga, info.colonna);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(n);
}


void rubrica_scheda_successiva()
{
  guint n, tot;
  GtkCList *clist;
  RubricaItem *i;
  InfoRubrica info;
  
  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  tot = rubrica_vista_get_indirizzi();
  n = rubrica_get_scheda_corrente()+1;
  
  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(vista.totale_indirizzi > 0);
  g_return_if_fail(n <= tot);

  i = g_list_nth_data(rubrica,n);

  g_return_if_fail(i != NULL);

  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), info.riga, info.colonna);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(n);
}




/*  ------------------------------------------------------------------ */

inline void rubrica_set_mostra_path(gboolean stato)
{
  mostra_path = stato;
}

inline gboolean rubrica_get_mostra_path(void)
{
  return (gboolean) mostra_path;
}

inline gboolean rubrica_is_empty(void)
{
  return (gboolean) rubrica_empty;
}

void rubrica_set_empty(gboolean stato)
{
  rubrica_empty = stato;
}











